# Run optimisation of marine reserve placement for fishery in Tun Mustapha Park, Malaysia
# Using static and dynamic models
# CJ Brown 10 Jan 2014
#Supplementary file for:
# Christopher J. Brown, Crow White, Maria Beger, Hedley S. Grantham, Benjamin S. Halpern, Carissa J. Klein, Peter J. Mumby, Vivitskaia J.D. Tulloch, Mary Ruckelshaus and Hugh P. Possingham. Fisheries and biodiversity benefits of using static versus dynamic models for designing marine reserve networks. Ecosphere
#
# You need to have function and type declarations .jl files in your current directory.
# Also the parameter specification file.
#For Julia version 0.3.2

#### Notes
#Using discount rates: profits are summed over the 'endhyear' period, and present value each year
#is calculated using the discount rate.
#Therefore, if using a discrate>0 set the endhyear period to the entire model period (e.g. yrsum = maxyears-2).
#Ensure the model is beggining at equilibrium (with no MPA).
#If you to optimise MPAs for equilibrium harvest, set discount rate to zero and chose an endhyear period
# that is suitably far into the future (e.g. 65 years). Set the years profits are summed over to zero
#(yrsum=0) if you want equilibrium profits.
#
#Parameters below will run optimisations for four scenarios - short-term profits (discounted) vs. equilbrium profits
#Crossed with fishing mortality at Fmsy and fishing mortality 2x Fmsy

using Gaston

set_terminal("aqua")
#using Debug

require("FishMod_types.jl")
reload("Fishmod_Functions.jl")
reload("Opt_Functions.jl")

##########
#Parameers for main scenario in paper
reload("FishParams_TMP.jl")
ninit = readcsv("InitCond_TMP_60.0.csv")
thisfilenam = "results/"

##########
# Optimisation parameters

##### Set boats and generate cost layer
bmin = 0.0
bmax = run_params.ncells * sp_params.popn.q/4
boat_opt_params = BOptParams(bmin,bmax, 0.001,10)

#Calculate effort that maximises profit
bopt_res = GSmultispp(boat_opt_params,run_params, fishery_params, sp_params,initparams, ninit)

#Dynamic opt params
kmax = 2000
kmin = 500
MAwin = 10 #moving average window
errtol = 0.00001
dynoptparams = DynOptParams(kmax, kmin, MAwin, errtol)

#### Set parameters and file names for multirun
maxyears = 65 #years to run model for
nmultiruns = 4
scnrnames = ["bhigh_long","bopt_long","bhigh_short","bopt_short"]
Fmult = [2. 1. 2. 1.] #multiplier for fishing effort
discrateruns = [0. 0. 0.1 0.1] #discount rates
yrmsumruns = [0 0 maxyears-2 maxyears-2] #years

for imultiruns = 1:nmultiruns
brun = copy(bopt_res.bopt).*Fmult[imultiruns]

yrsum = yrmsumruns[imultiruns]

######################################################################
#make new fishery run parameters
fish_runparams = FisheryParams(brun, fishery_params) #btot agefish, mpacells
run_params.endhyears = (maxyears-yrsum):maxyears
run_params.discrate = discrateruns[imultiruns]

modres_base = fishmod(run_params, fish_runparams, sp_params,initparams, ninit)
#reset ninit after model has been run to equilibrium for this level of fishing
ninit = modres_base.nvals

#calculate cost layer, as discounted profits over full time-period.
#Must make sure that model begins at equilibruim
costlyr = CalcCostLyr(run_params, fish_runparams, sp_params,initparams, ninit)

figure(2)
valmat = matrix_values_map(grefsallF, grefs, costlyr, -1.)
imagesc(valmat,"title","Cost layer")

#save cost layer
writecsv(string(thisfilenam,"best mpas/TMPCostLayer_",scnrnames[imultiruns],".csv"),costlyr)

######################################################################
####### Loop dynamic greedy algorithm
######################################################################
nconstarg = 10
constargvals = linspace(0.0, 0.70, nconstarg)
dynsave = fill(0., nconstarg)
statsave = fill(0., nconstarg)
naivesave = fill(0., nconstarg)

#Save constargvals
writecsv(string(thisfilenam,"best mpas/ConsTargVals.csv"), constargvals)

## Reset run params with number of years
run_params_new = deepcopy(run_params)
run_params_new.nyears = maxyears
run_params_new.endhyears = (maxyears-yrsum):maxyears

for icons =1:nconstarg
    constarg = ConsTarg(constargvals[icons], hab_map_params.nhabcells,20.)

    staticres = static_greedy(constarg, hab_map_params, costlyr,run_params_new, fish_runparams, sp_params,initparams, ninit)

    fishp_static = FisheryParams(fish_runparams, staticres.optmpacells)
    modres_static = fishmod(run_params_new, fishp_static, sp_params,initparams, ninit)

    #Dynamic, set starting size
    #Works best if it starts bigger than target area
        startszs = convert(Int64, round(min([constarg.aprotect+0.2, 0.9]) * run_params.ncells))

     if startszs<=0.
        start_mpa = []
    else
        start_mpa = randsample(1:run_params_new.ncells, startszs)
    end
    #use static as starting point if in a short term scenario
    if yrmsumruns[imultiruns]>0
        start_mpa = staticres.optmpacells
    end
    fishp_dynstart = FisheryParams(fish_runparams,start_mpa)  #set starting condition for dynamic

    dynamicres = dynamic_greedy(run_params_new, fishp_dynstart, sp_params,hab_map_params,initparams,
ninit, constarg, bopt_res, dynoptparams)

    plot(1:length(dynamicres.mavals), dynamicres.mavals, 1:length(dynamicres.esave), dynamicres.esave)

    fishp_dyn = FisheryParams(fish_runparams, dynamicres.bestfishparams.mpacells)

    modres_dynamic = fishmod(run_params_new, fishp_dyn, sp_params,initparams, ninit)

    #Save best MPA to file
    flnmd = strcat(thisfilenam,"best mpas/Dynamic_",scnrnames[imultiruns],round(constarg.aprotect,2),".csv")
    flnms = strcat(thisfilenam,"best mpas/Static_",scnrnames[imultiruns],round(constarg.aprotect,2),".csv")
    writecsv(flnmd, dynamicres.bestfishparams.mpacells)
    writecsv(flnms, staticres.optmpacells)
    #Save results
    dynsave[icons] = dynamicres.bestmodres.profitend/sum(costlyr)
    statsave[icons] = staticres.staticharv/sum(costlyr)
    naivesave[icons] = staticres.naiveharv/sum(costlyr)

    ## Print results
    println("Dynamic best - saved")
    println(dynamicres.bestmodres.profitend/sum(costlyr))

    println("Static best - saved")
    println(staticres.staticharv/sum(costlyr))

    # Habitat representation
    println("Dynamic best - hab representation")
    println(calchabrep(fishp_dyn, hab_map_params))

    println("Static best - hab representation")
    println(calchabrep(fishp_static, hab_map_params))

    println("Conservation targets")
    println(constarg)


    #plot mpa designs
    #figure(icons+ nconstarg)
    #mpamat = matrix_mpa_map(grefsallF, grefs, staticres.optmpacells)
    #imagesc(mpamat,"title",string("Best static reserve, CTarg = ",constargvals[icons]))

    #figure(icons + 2*nconstarg)
    #plot dynamic MPA
    #optdynmpacells = find(dynamicres.bestfishparams.is_fished.==0)
    #mpamat = matrix_mpa_map(grefsallF, grefs, optdynmpacells)
    #imagesc(mpamat,"title",string("Best dynamic reserve, CTarg = ",constargvals[icons]))

    #figure(icons)
    #plot([1:kmax], dynamicres.esave,"color","blue",
#[1:kmax], dynamicres.mavals,
 #"xlabel","Runs", "ylabel", "Obj func value","title",string("Energy values, CTarg = ",constargvals[icons]))

end


figure(6)
plot(constargvals, dynsave ,"plotstyle","points","pointsize",2,"legend","Dynamic"
,constargvals, statsave,"plotstyle","points","pointsize",2,"legend","Static"
,constargvals, naivesave,"plotstyle","points","pointsize",2,"legend","Naive static")

constargvals


#### Save results
println(dynsave)

writecsv(string(thisfilenam, "dynsave_", scnrnames[imultiruns],".csv"), dynsave)
writecsv(string(thisfilenam, "statsave_", scnrnames[imultiruns],".csv"), statsave)
writecsv(string(thisfilenam, "naivesave_", scnrnames[imultiruns],".csv"), naivesave)
writecsv(string(thisfilenam, "constargvals_", scnrnames[imultiruns],".csv"), constargvals)

end
